/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file SchedulerFrontend.hh
 *
 * Declaration of SchedulerFrontend class.
 *
 * @author Ari Metsähalme 2005 (ari.metsahalme-no.spam-tut.fi)
 * @author Vladimír Guzma 2008 (vladimir.guzma-no.spam-tut.fi)
 * @note rating: red
 */

#ifndef TTA_SCHEDULER_FRONTEND_HH
#define TTA_SCHEDULER_FRONTEND_HH

#include "Exception.hh"
#include "SchedulerPluginLoader.hh"
#include "SchedulingPlan.hh"
#include "OperationPool.hh"
#include "UniversalMachine.hh"

class SchedulerCmdLineOptions;
class BaseSchedulerModule;
class UniversalMachine;

namespace TTAProgram {
    class Program;
}

namespace TTAMachine {
    class Machine;
}

namespace TPEF {
    class Binary;
}

class InterPassData;

/**
 * The top-level component of the Scheduler that the clients use to
 * setup and launch the scheduling process.
 */
class SchedulerFrontend {
public:
    SchedulerFrontend();
    virtual ~SchedulerFrontend();

    void schedule(SchedulerCmdLineOptions& options)
        throw (Exception);
    TTAProgram::Program* schedule(
        const TTAProgram::Program& source, const TTAMachine::Machine& target,
        const SchedulingPlan& schedulingPlan,
        InterPassData* interPassData = NULL)
        throw (Exception);
private:
    /// Copying forbidden.
    SchedulerFrontend(const SchedulerFrontend&);
    /// Assignment forbidden.
    SchedulerFrontend& operator=(const SchedulerFrontend&);
    void prepareModule(
        BaseSchedulerModule& module, TTAProgram::Program& source,
        const TTAMachine::Machine& target)
        throw (IOException, Exception);

    SchedulerPluginLoader* pluginLoader_;
};

#endif
