/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file StartableSchedulerModule.cc
 *
 * Implementation of StartableSchedulerModule class.
 *
 * @author Ari Mets�halme 2005 (ari.metsahalme-no.spam-tut.fi)
 * @note rating: red
 */

#include "StartableSchedulerModule.hh"

/**
 * Constructor.
 */
StartableSchedulerModule::StartableSchedulerModule(): BaseSchedulerModule() {
}

/**
 * Destructor.
 */
StartableSchedulerModule::~StartableSchedulerModule() {
}

/**
 * Return true if this module can be run independently.
 *
 * Startable does not mean that, if started, the module won't fail due
 * to some failure conditions.
 *
 * @return True always, because this type of module is supposed to be run
 * independently.
 */
bool
StartableSchedulerModule::isStartable() const {
    return true;
}

/**
 * A short description of the module.
 *
 * @return The description as a string.
 */   
std::string
StartableSchedulerModule::shortDescription() const {
    return "Generic startable module. Detailed description is missing.";
}

/**
 * Optional longer description of the Module.
 *
 * This description can include usage instructions, details of choice of
 * helper modules, etc.
 *
 * @return The description as a string.
 */
std::string
StartableSchedulerModule::longDescription() const {
    return "Generic startable module. Detailed description is missing.";
}
