/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file InstructionExecution.hh
 *
 * Declaration of InstructionExecution class.
 *
 * @author Pekka Jskelinen 2004 (pjaaskel-no.spam-cs.tut.fi)
 */

#ifndef TTA_INSTRUCTION_EXECUTION_HH
#define TTA_INSTRUCTION_EXECUTION_HH
 
#include "Exception.hh"
#include "ExecutionTrace.hh"
#include "RelationalDBQueryResult.hh"
#include "SimulatorConstants.hh"

/**
 * Class used to navigate through the list of all execution cycles.
 *
 * This class is also used to access the data of the pointed record.
 */
class InstructionExecution {
public:
    InstructionExecution(RelationalDBQueryResult* result);
    virtual ~InstructionExecution();

    ClockCycleCount cycle() const throw (NotAvailable);
    InstructionAddress address() const throw (NotAvailable);

    void next() throw (NotAvailable);
    bool hasNext() const;

private:
    RelationalDBQueryResult* result_;
    int addressColumnIndex_;
    int cycleColumnIndex_;
};

#endif
