/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file DropDownMenu.hh
 *
 * Declaration of DropDownMenu class.
 *
 * @author Jussi Nyknen 2004 (nykanen-no.spam-cs.tut.fi)
 * @note rating: red
 */

#ifndef TTA_DROP_DOWN_MENU_HH
#define TTA_DROP_DOWN_MENU_HH

#include <wx/wx.h>

/**
 * Class that models drop down menu.
 *
 * Drop down menu is shown, when user clicks right mouse button over
 * tree item or list item in main window.
 */
class DropDownMenu : public wxMenu {
public:

    /**
     * Type of the drop down menu.
     */
    enum MenuType {
        MENU_PATH,      ///< Path menu.
        MENU_MODULE,    ///< Module menu.
        MENU_OPERATION  ///< Operation menu.
    };

    explicit DropDownMenu(MenuType type);
    virtual ~DropDownMenu();

private:
    /// Copying not allowed.
    DropDownMenu(const DropDownMenu&);
    /// Assignment not allowed.
    DropDownMenu& operator=(const DropDownMenu&);
    
    void onMenuEvent(wxCommandEvent& event);
    void updateMenu();

    DECLARE_EVENT_TABLE()
};

#endif
